Package: qs
Type: Package
Title: Quick Serialization of R Objects
Version: 0.27.2
Date: 2024-09-30
Authors@R: c(
    person("Travers", "Ching", email = "traversc@gmail.com", role = c("aut", "cre", "cph")),
    person("Yann", "Collet", role = c("ctb", "cph"), comment = "Yann Collet is the author of the bundled zstd, lz4 and xxHash code"),
    person("Facebook, Inc.", role = "cph", comment = "Facebook is the copyright holder of the bundled zstd code"),
    person("Reichardt", "Tino", role = c("ctb", "cph"), comment = "Contributor/copyright holder of zstd bundled code"),
    person("Skibinski", "Przemyslaw", role = c("ctb", "cph"), comment = "Contributor/copyright holder of zstd bundled code"),
    person("Mori", "Yuta", role = c("ctb", "cph"), comment = "Contributor/copyright holder of zstd bundled code"),
    person("Romain", "Francois", role = c("ctb", "cph"), comment = "Derived example/tutorials for ALTREP structures"),
    person("Francesc", "Alted", role = c("ctb", "cph"), comment = "Shuffling routines derived from Blosc library"),
    person("Bryce", "Chamberlain", email = "superchordate@gmail.com", role = c("ctb"), comment = "qsavem and qload functions"),
    person("Salim", "Brüggemann", email = "salim-b@pm.me", role = c("ctb"), comment = "Contributing to documentation (ORCID:0000-0002-5329-5987)"))
Maintainer: Travers Ching <traversc@gmail.com>
Description: Provides functions for quickly writing and reading any R object to and from disk.  
License: GPL-3
LazyData: true
Biarch: true
Depends: R (>= 3.0.2)
Imports: Rcpp, RApiSerialize (>= 0.1.4), stringfish (>= 0.15.1)
LinkingTo: Rcpp, RApiSerialize, stringfish, BH
Encoding: UTF-8
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, testthat, dplyr, data.table
VignetteBuilder: knitr
Copyright: This package includes code from the 'zstd' library owned by
        Facebook, Inc. and created by Yann Collet; the 'lz4' library
        created and owned by Yann Collet; xxHash library created and
        owned by Yann Collet; and code derived from the 'Blosc' library
        created and owned by Francesc Alted.
URL: https://github.com/qsbase/qs
BugReports: https://github.com/qsbase/qs/issues
NeedsCompilation: yes
Packaged: 2024-09-30 16:54:03 UTC; tching
Author: Travers Ching [aut, cre, cph],
  Yann Collet [ctb, cph] (Yann Collet is the author of the bundled zstd,
    lz4 and xxHash code),
  Facebook, Inc. [cph] (Facebook is the copyright holder of the bundled
    zstd code),
  Reichardt Tino [ctb, cph] (Contributor/copyright holder of zstd bundled
    code),
  Skibinski Przemyslaw [ctb, cph] (Contributor/copyright holder of zstd
    bundled code),
  Mori Yuta [ctb, cph] (Contributor/copyright holder of zstd bundled
    code),
  Romain Francois [ctb, cph] (Derived example/tutorials for ALTREP
    structures),
  Francesc Alted [ctb, cph] (Shuffling routines derived from Blosc
    library),
  Bryce Chamberlain [ctb] (qsavem and qload functions),
  Salim Brüggemann [ctb] (Contributing to documentation
    (ORCID:0000-0002-5329-5987))
Repository: CRAN
Date/Publication: 2024-10-01 22:10:02 UTC
Built: R 4.4.1; aarch64-apple-darwin21.6.0; 2024-11-06 04:01:14 UTC; unix
