Package: qs
Type: Package
Title: Quick Serialization of R Objects
Version: 0.23.5
Date: 2021-1-5
Authors@R: c(
    person("Travers", "Ching", email = "traversc@gmail.com", role = c("aut", "cre", "cph")),
    person("Yann", "Collet", role = c("ctb", "cph"), comment = "Yann Collet is the author of the bundled zstd, lz4 and xxHash code"),
    person("Facebook, Inc.", role = "cph", comment = "Facebook is the copyright holder of the bundled zstd code"),
    person("Reichardt", "Tino", role = c("ctb", "cph"), comment = "Contributor/copyright holder of zstd bundled code"),
    person("Skibinski", "Przemyslaw", role = c("ctb", "cph"), comment = "Contributor/copyright holder of zstd bundled code"),
    person("Mori", "Yuta", role = c("ctb", "cph"), comment = "Contributor/copyright holder of zstd bundled code"),
    person("Romain", "Francois", role = c("ctb", "cph"), comment = "Derived example/tutorials for ALTREP structures"),
    person("Francesc", "Alted", role = c("ctb", "cph"), comment = "Shuffling routines derived from Blosc library"),
    person("Bryce", "Chamberlain", email = "superchordate@gmail.com", role = c("ctb"), comment = "qsavem and qload functions"))
Maintainer: Travers Ching <traversc@gmail.com>
Description: Provides functions for quickly writing and reading any R object to and from disk.  
License: GPL-3 | file LICENSE
LazyData: true
Biarch: true
Depends: R (>= 3.5.0)
SystemRequirements: C++11
Imports: Rcpp, RApiSerialize, stringfish (>= 0.14.1)
LinkingTo: Rcpp, RApiSerialize, stringfish
RoxygenNote: 7.1.0
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Copyright: This package includes code from the 'zstd' library owned by
        Facebook, Inc. and created by Yann Collet; the 'lz4' library
        created and owned by Yann Collet; xxHash library created and
        owned by Yann Collet; and code derived from the 'Blosc' library
        created and owned by Francesc Alted.
URL: https://github.com/traversc/qs
BugReports: https://github.com/traversc/qs/issues
NeedsCompilation: yes
Packaged: 2021-01-06 18:08:08 UTC; tching
Author: Travers Ching [aut, cre, cph],
  Yann Collet [ctb, cph] (Yann Collet is the author of the bundled zstd,
    lz4 and xxHash code),
  Facebook, Inc. [cph] (Facebook is the copyright holder of the bundled
    zstd code),
  Reichardt Tino [ctb, cph] (Contributor/copyright holder of zstd bundled
    code),
  Skibinski Przemyslaw [ctb, cph] (Contributor/copyright holder of zstd
    bundled code),
  Mori Yuta [ctb, cph] (Contributor/copyright holder of zstd bundled
    code),
  Romain Francois [ctb, cph] (Derived example/tutorials for ALTREP
    structures),
  Francesc Alted [ctb, cph] (Shuffling routines derived from Blosc
    library),
  Bryce Chamberlain [ctb] (qsavem and qload functions)
Repository: CRAN
Date/Publication: 2021-01-06 18:50:03 UTC
Built: R 4.0.3; x86_64-w64-mingw32; 2021-01-06 21:03:04 UTC; windows
Archs: x64
