# Generated by roxygen2: do not edit by hand

export(base85_decode)
export(base85_encode)
export(base91_decode)
export(base91_encode)
export(blosc_shuffle_raw)
export(blosc_unshuffle_raw)
export(catquo)
export(decode_source)
export(encode_source)
export(get_altrep_class_info)
export(is_big_endian)
export(lz4_compress_bound)
export(lz4_compress_raw)
export(lz4_decompress_raw)
export(qattributes)
export(qcache)
export(qdeserialize)
export(qdump)
export(qload)
export(qread)
export(qread_fd)
export(qread_handle)
export(qread_ptr)
export(qread_url)
export(qreadm)
export(qsave)
export(qsave_fd)
export(qsave_handle)
export(qsavem)
export(qserialize)
export(register_altrep_class)
export(set_trust_promises)
export(unregister_altrep_class)
export(zstd_compress_bound)
export(zstd_compress_raw)
export(zstd_decompress_raw)
import(RApiSerialize)
import(stringfish)
importFrom(Rcpp,evalCpp)
useDynLib(qs, .registration = TRUE)
