import(RApiSerialize)
import(stringfish)
importFrom(Rcpp,sourceCpp)
useDynLib(qs, .registration = TRUE)
export("qsave", 
       "qread", 
       "qsave_fd", 
       "qread_fd", 
       "qsave_handle", 
       "qread_handle", 
       "qserialize",
       "qdeserialize",
       "qread_ptr", 
       "qdump", 
       "zstd_compress_bound", 
       "zstd_compress_raw", 
       "zstd_decompress_raw", 
       "lz4_compress_bound", 
       "lz4_compress_raw", 
       "lz4_decompress_raw", 
       "is_big_endian", 
       "randomStrings", 
       "convertToAlt", 
       "blosc_shuffle_raw", 
       "blosc_unshuffle_raw",
       "base85_encode", 
       "base85_decode",
       "base91_encode", 
       "base91_decode",
       "catquo",
       "qsavem",
       "qreadm",
       "qload")
